@@ -79,8 +79,6 @@
 #ifndef __inline__
 #if defined(__GNUC__)
     /* gcc has __inline__ */
-#elif defined(__HIGHC__)
-#define __inline__ _Inline
 #else
 #define __inline__ /**/
 #endif
@@ -88,8 +86,6 @@
 #ifndef __inline
 #if defined(__GNUC__)
     /* gcc has __inline */
-#elif defined(__HIGHC__)
-#define __inline _Inline
 #else
 #define __inline /**/
 #endif
@@ -98,11 +94,13 @@
 #if !defined(__GNUC__) && !defined(__FUNCTION__)
 #define __FUNCTION__ __func__   /* C99 */
 #endif
-#if defined(NO_INLINE) || defined(DO_PROTOTYPES)
+
+#if defined(DO_PROTOTYPES)
 #if !defined(__arm__)
-#if !defined(__sparc__) && !defined(__sparc) && !defined(__arm32__) && !defined(__nds32__) \
+#if !defined(__sparc__) && !defined(__arm32__) && !defined(__nds32__) \
       && !(defined(__alpha__) && defined(linux)) \
       && !(defined(__ia64__) && defined(linux)) \
+      && !(defined(__mips64) && defined(linux)) \
 
 extern _X_EXPORT void outb(unsigned short, unsigned char);
 extern _X_EXPORT void outw(unsigned short, unsigned short);
@@ -119,27 +117,23 @@ extern _X_EXPORT unsigned int inb(unsigned long);
 extern _X_EXPORT unsigned int inw(unsigned long);
 extern _X_EXPORT unsigned int inl(unsigned long);
 
+#ifdef __SUNPRO_C
+extern _X_EXPORT unsigned char  xf86ReadMmio8    (void *, unsigned long);
+extern _X_EXPORT unsigned short xf86ReadMmio16Be (void *, unsigned long);
+extern _X_EXPORT unsigned short xf86ReadMmio16Le (void *, unsigned long);
+extern _X_EXPORT unsigned int   xf86ReadMmio32Be (void *, unsigned long);
+extern _X_EXPORT unsigned int   xf86ReadMmio32Le (void *, unsigned long);
+extern _X_EXPORT void xf86WriteMmio8    (void *, unsigned long, unsigned int);
+extern _X_EXPORT void xf86WriteMmio16Be (void *, unsigned long, unsigned int);
+extern _X_EXPORT void xf86WriteMmio16Le (void *, unsigned long, unsigned int);
+extern _X_EXPORT void xf86WriteMmio32Be (void *, unsigned long, unsigned int);
+extern _X_EXPORT void xf86WriteMmio32Le (void *, unsigned long, unsigned int);
+#endif                          /* _SUNPRO_C */
 #endif                          /* __sparc__,  __arm32__, __alpha__, __nds32__ */
 #endif                          /* __arm__ */
 
-#if defined(__powerpc__) && !defined(__OpenBSD__)
-extern unsigned long ldq_u(unsigned long *);
-extern unsigned long ldl_u(unsigned int *);
-extern unsigned long ldw_u(unsigned short *);
-extern void stq_u(unsigned long, unsigned long *);
-extern void stl_u(unsigned long, unsigned int *);
-extern void stw_u(unsigned long, unsigned short *);
-extern void mem_barrier(void);
-extern void write_mem_barrier(void);
-extern void stl_brx(unsigned long, volatile unsigned char *, int);
-extern void stw_brx(unsigned short, volatile unsigned char *, int);
-extern unsigned long ldl_brx(volatile unsigned char *, int);
-extern unsigned short ldw_brx(volatile unsigned char *, int);
-#endif                          /* __powerpc__ && !__OpenBSD */
-
 #endif                          /* NO_INLINE || DO_PROTOTYPES */
 
-#ifndef NO_INLINE
 #ifdef __GNUC__
 #ifdef __i386__
 
@@ -192,14 +186,7 @@ extern unsigned short ldw_brx(volatile unsigned char *, int);
 
 #elif defined __powerpc__
 
-#if defined(linux) && defined(__powerpc64__)
-#include <linux/version.h>
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
-#include <asm/memory.h>
-#endif
-#endif                          /* defined(linux) && defined(__powerpc64__) */
-
-#ifndef eieio                   /* We deal with arch-specific eieio() routines above... */
+#ifndef eieio
 #define eieio() __asm__ __volatile__ ("eieio" ::: "memory")
 #endif                          /* eieio */
 #define mem_barrier()	eieio()
@@ -212,7 +199,10 @@ extern unsigned short ldw_brx(volatile unsigned char *, int);
 #define write_mem_barrier()     /* XXX: nop for now */
 #endif
 #endif                          /* __GNUC__ */
-#endif                          /* NO_INLINE */
+
+#ifndef barrier
+#define barrier()
+#endif
 
 #ifndef mem_barrier
 #define mem_barrier()           /* NOP */
@@ -222,133 +212,8 @@ extern unsigned short ldw_brx(volatile unsigned char *, int);
 #define write_mem_barrier()     /* NOP */
 #endif
 
-#ifndef NO_INLINE
 #ifdef __GNUC__
-
-/* Define some packed structures to use with unaligned accesses */
-
-struct __una_u64 {
-    uint64_t x __attribute__ ((packed));
-};
-struct __una_u32 {
-    uint32_t x __attribute__ ((packed));
-};
-struct __una_u16 {
-    uint16_t x __attribute__ ((packed));
-};
-
-/* Elemental unaligned loads */
-
-static __inline__ uint64_t
-ldq_u(uint64_t * p)
-{
-    const struct __una_u64 *ptr = (const struct __una_u64 *) p;
-
-    return ptr->x;
-}
-
-static __inline__ uint32_t
-ldl_u(uint32_t * p)
-{
-    const struct __una_u32 *ptr = (const struct __una_u32 *) p;
-
-    return ptr->x;
-}
-
-static __inline__ uint16_t
-ldw_u(uint16_t * p)
-{
-    const struct __una_u16 *ptr = (const struct __una_u16 *) p;
-
-    return ptr->x;
-}
-
-/* Elemental unaligned stores */
-
-static __inline__ void
-stq_u(uint64_t val, uint64_t * p)
-{
-    struct __una_u64 *ptr = (struct __una_u64 *) p;
-
-    ptr->x = val;
-}
-
-static __inline__ void
-stl_u(uint32_t val, uint32_t * p)
-{
-    struct __una_u32 *ptr = (struct __una_u32 *) p;
-
-    ptr->x = val;
-}
-
-static __inline__ void
-stw_u(uint16_t val, uint16_t * p)
-{
-    struct __una_u16 *ptr = (struct __una_u16 *) p;
-
-    ptr->x = val;
-}
-#else                           /* !__GNUC__ */
-
-#include <string.h>             /* needed for memmove */
-
-static __inline__ uint64_t
-ldq_u(uint64_t * p)
-{
-    uint64_t ret;
-
-    memmove(&ret, p, sizeof(*p));
-    return ret;
-}
-
-static __inline__ uint32_t
-ldl_u(uint32_t * p)
-{
-    uint32_t ret;
-
-    memmove(&ret, p, sizeof(*p));
-    return ret;
-}
-
-static __inline__ uint16_t
-ldw_u(uint16_t * p)
-{
-    uint16_t ret;
-
-    memmove(&ret, p, sizeof(*p));
-    return ret;
-}
-
-static __inline__ void
-stq_u(uint64_t val, uint64_t * p)
-{
-    uint64_t tmp = val;
-
-    memmove(p, &tmp, sizeof(*p));
-}
-
-static __inline__ void
-stl_u(uint32_t val, uint32_t * p)
-{
-    uint32_t tmp = val;
-
-    memmove(p, &tmp, sizeof(*p));
-}
-
-static __inline__ void
-stw_u(uint16_t val, uint16_t * p)
-{
-    uint16_t tmp = val;
-
-    memmove(p, &tmp, sizeof(*p));
-}
-
-#endif                          /* __GNUC__ */
-#endif                          /* NO_INLINE */
-
-#ifndef NO_INLINE
-#ifdef __GNUC__
-#if (defined(linux) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)) && (defined(__alpha__))
+#if defined(__alpha__)
 
 #ifdef linux
 /* for Linux on Alpha, we use the LIBC _inx/_outx routines */
@@ -421,7 +286,7 @@ extern _X_EXPORT unsigned int inl(unsigned int port);
 #include <machine/pio.h>
 #endif                          /* __NetBSD__ */
 
-#elif (defined(linux) || defined(__FreeBSD__)) && defined(__amd64__)
+#elif defined(__amd64__)
 
 #include <inttypes.h>
 
@@ -470,7 +335,7 @@ inl(unsigned short port)
     return ret;
 }
 
-#elif (defined(linux) || defined(sun) || defined(__OpenBSD__) || defined(__FreeBSD__)) && defined(__sparc__)
+#elif defined(__sparc__)
 
 #ifndef ASI_PL
 #define ASI_PL 0x88
@@ -653,58 +518,8 @@ xf86WriteMmio32Le(__volatile__ void *base, const unsigned long offset,
     barrier();
 }
 
-static __inline__ void
-xf86WriteMmio8NB(__volatile__ void *base, const unsigned long offset,
-                 const unsigned int val)
-{
-    unsigned long addr = ((unsigned long) base) + offset;
-
-    __asm__ __volatile__("stba %0, [%1] %2":    /* No outputs */
-                         :"r"(val), "r"(addr), "i"(ASI_PL));
-}
-
-static __inline__ void
-xf86WriteMmio16BeNB(__volatile__ void *base, const unsigned long offset,
-                    const unsigned int val)
-{
-    unsigned long addr = ((unsigned long) base) + offset;
-
-    __asm__ __volatile__("sth %0, [%1]":        /* No outputs */
-                         :"r"(val), "r"(addr));
-}
-
-static __inline__ void
-xf86WriteMmio16LeNB(__volatile__ void *base, const unsigned long offset,
-                    const unsigned int val)
-{
-    unsigned long addr = ((unsigned long) base) + offset;
-
-    __asm__ __volatile__("stha %0, [%1] %2":    /* No outputs */
-                         :"r"(val), "r"(addr), "i"(ASI_PL));
-}
-
-static __inline__ void
-xf86WriteMmio32BeNB(__volatile__ void *base, const unsigned long offset,
-                    const unsigned int val)
-{
-    unsigned long addr = ((unsigned long) base) + offset;
-
-    __asm__ __volatile__("st %0, [%1]": /* No outputs */
-                         :"r"(val), "r"(addr));
-}
-
-static __inline__ void
-xf86WriteMmio32LeNB(__volatile__ void *base, const unsigned long offset,
-                    const unsigned int val)
-{
-    unsigned long addr = ((unsigned long) base) + offset;
-
-    __asm__ __volatile__("sta %0, [%1] %2":     /* No outputs */
-                         :"r"(val), "r"(addr), "i"(ASI_PL));
-}
-
 #elif defined(__mips__) || (defined(__arm32__) && !defined(__linux__))
-#ifdef __arm32__
+#if defined(__arm32__) || defined(__mips64)
 #define PORT_SIZE long
 #else
 #define PORT_SIZE short
@@ -782,7 +597,7 @@ xf86WriteMmio32Be(__volatile__ void *base, const unsigned long offset,
 #endif                          /* !linux */
 #endif                          /* __mips__ */
 
-#elif (defined(linux) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__FreeBSD__)) && defined(__powerpc__)
+#elif defined(__powerpc__)
 
 #ifndef MAP_FAILED
 #define MAP_FAILED ((void *)-1)
@@ -841,60 +656,13 @@ xf86ReadMmio32Le(__volatile__ void *base, const unsigned long offset)
 }
 
 static __inline__ void
-xf86WriteMmioNB8(__volatile__ void *base, const unsigned long offset,
-                 const unsigned char val)
+xf86WriteMmio8(__volatile__ void *base, const unsigned long offset,
+               const unsigned char val)
 {
     __asm__
         __volatile__("stbx %1,%2,%3\n\t":"=m"
                      (*((volatile unsigned char *) base + offset))
                      :"r"(val), "b"(base), "r"(offset));
-}
-
-static __inline__ void
-xf86WriteMmioNB16Le(__volatile__ void *base, const unsigned long offset,
-                    const unsigned short val)
-{
-    __asm__
-        __volatile__("sthbrx %1,%2,%3\n\t":"=m"
-                     (*((volatile unsigned char *) base + offset))
-                     :"r"(val), "b"(base), "r"(offset));
-}
-
-static __inline__ void
-xf86WriteMmioNB16Be(__volatile__ void *base, const unsigned long offset,
-                    const unsigned short val)
-{
-    __asm__
-        __volatile__("sthx %1,%2,%3\n\t":"=m"
-                     (*((volatile unsigned char *) base + offset))
-                     :"r"(val), "b"(base), "r"(offset));
-}
-
-static __inline__ void
-xf86WriteMmioNB32Le(__volatile__ void *base, const unsigned long offset,
-                    const unsigned int val)
-{
-    __asm__
-        __volatile__("stwbrx %1,%2,%3\n\t":"=m"
-                     (*((volatile unsigned char *) base + offset))
-                     :"r"(val), "b"(base), "r"(offset));
-}
-
-static __inline__ void
-xf86WriteMmioNB32Be(__volatile__ void *base, const unsigned long offset,
-                    const unsigned int val)
-{
-    __asm__
-        __volatile__("stwx %1,%2,%3\n\t":"=m"
-                     (*((volatile unsigned char *) base + offset))
-                     :"r"(val), "b"(base), "r"(offset));
-}
-
-static __inline__ void
-xf86WriteMmio8(__volatile__ void *base, const unsigned long offset,
-               const unsigned char val)
-{
-    xf86WriteMmioNB8(base, offset, val);
     eieio();
 }
 
@@ -902,7 +670,10 @@ static __inline__ void
 xf86WriteMmio16Le(__volatile__ void *base, const unsigned long offset,
                   const unsigned short val)
 {
-    xf86WriteMmioNB16Le(base, offset, val);
+    __asm__
+        __volatile__("sthbrx %1,%2,%3\n\t":"=m"
+                     (*((volatile unsigned char *) base + offset))
+                     :"r"(val), "b"(base), "r"(offset));
     eieio();
 }
 
@@ -910,7 +681,10 @@ static __inline__ void
 xf86WriteMmio16Be(__volatile__ void *base, const unsigned long offset,
                   const unsigned short val)
 {
-    xf86WriteMmioNB16Be(base, offset, val);
+    __asm__
+        __volatile__("sthx %1,%2,%3\n\t":"=m"
+                     (*((volatile unsigned char *) base + offset))
+                     :"r"(val), "b"(base), "r"(offset));
     eieio();
 }
 
@@ -918,7 +692,10 @@ static __inline__ void
 xf86WriteMmio32Le(__volatile__ void *base, const unsigned long offset,
                   const unsigned int val)
 {
-    xf86WriteMmioNB32Le(base, offset, val);
+    __asm__
+        __volatile__("stwbrx %1,%2,%3\n\t":"=m"
+                     (*((volatile unsigned char *) base + offset))
+                     :"r"(val), "b"(base), "r"(offset));
     eieio();
 }
 
@@ -926,7 +703,10 @@ static __inline__ void
 xf86WriteMmio32Be(__volatile__ void *base, const unsigned long offset,
                   const unsigned int val)
 {
-    xf86WriteMmioNB32Be(base, offset, val);
+    __asm__
+        __volatile__("stwx %1,%2,%3\n\t":"=m"
+                     (*((volatile unsigned char *) base + offset))
+                     :"r"(val), "b"(base), "r"(offset));
     eieio();
 }
 
@@ -1012,12 +792,9 @@ xf_outl(unsigned short port, unsigned int val)
 
 /*
  * Assume all port access are aligned.  We need to revise this implementation
- * if there is unaligned port access.  For ldq_u, ldl_u, ldw_u, stq_u, stl_u and
- * stw_u, they are assumed unaligned.
+ * if there is unaligned port access.
  */
 
-#define barrier()               /* no barrier */
-
 #define PORT_SIZE long
 
 static __inline__ unsigned char
@@ -1034,13 +811,6 @@ xf86WriteMmio8(__volatile__ void *base, const unsigned long offset,
     barrier();
 }
 
-static __inline__ void
-xf86WriteMmio8NB(__volatile__ void *base, const unsigned long offset,
-                 const unsigned int val)
-{
-    *(volatile unsigned char *) ((unsigned char *) base + offset) = val;
-}
-
 static __inline__ unsigned short
 xf86ReadMmio16Swap(__volatile__ void *base, const unsigned long offset)
 {
@@ -1079,23 +849,6 @@ xf86WriteMmio16(__volatile__ void *base, const unsigned long offset,
     barrier();
 }
 
-static __inline__ void
-xf86WriteMmio16SwapNB(__volatile__ void *base, const unsigned long offset,
-                      const unsigned int val)
-{
-    unsigned long addr = ((unsigned long) base) + offset;
-
-    __asm__ __volatile__("wsbh %0, %0;\n\t" "shi %0, [%1];\n\t":        /* No outputs */
-                         :"r"(val), "r"(addr));
-}
-
-static __inline__ void
-xf86WriteMmio16NB(__volatile__ void *base, const unsigned long offset,
-                  const unsigned int val)
-{
-    *(volatile unsigned short *) ((unsigned char *) base + offset) = val;
-}
-
 static __inline__ unsigned int
 xf86ReadMmio32Swap(__volatile__ void *base, const unsigned long offset)
 {
@@ -1135,23 +888,6 @@ xf86WriteMmio32(__volatile__ void *base, const unsigned long offset,
     barrier();
 }
 
-static __inline__ void
-xf86WriteMmio32SwapNB(__volatile__ void *base, const unsigned long offset,
-                      const unsigned int val)
-{
-    unsigned long addr = ((unsigned long) base) + offset;
-
-    __asm__ __volatile__("wsbh %0, %0;\n\t" "rotri %0, %0, 16;\n\t" "swi %0, [%1];\n\t":        /* No outputs */
-                         :"r"(val), "r"(addr));
-}
-
-static __inline__ void
-xf86WriteMmio32NB(__volatile__ void *base, const unsigned long offset,
-                  const unsigned int val)
-{
-    *(volatile unsigned int *) ((unsigned char *) base + offset) = val;
-}
-
 #if defined(NDS32_MMIO_SWAP)
 static __inline__ void
 outb(unsigned PORT_SIZE port, unsigned char val)
@@ -1189,50 +925,6 @@ inl(unsigned PORT_SIZE port)
     return xf86ReadMmio32Swap(IOPortBase, port);
 }
 
-static __inline__ unsigned long
-ldq_u(unsigned long *p)
-{
-    unsigned long addr = (unsigned long) p;
-    unsigned int ret;
-
-    __asm__ __volatile__("lmw.bi %0, [%1], %0, 0;\n\t"
-                         "wsbh %0, %0;\n\t" "rotri %0, %0, 16;\n\t":"=r"(ret)
-                         :"r"(addr));
-
-    return ret;
-}
-
-static __inline__ unsigned long
-ldl_u(unsigned int *p)
-{
-    unsigned long addr = (unsigned long) p;
-    unsigned int ret;
-
-    __asm__ __volatile__("lmw.bi %0, [%1], %0, 0;\n\t"
-                         "wsbh %0, %0;\n\t" "rotri %0, %0, 16;\n\t":"=r"(ret)
-                         :"r"(addr));
-
-    return ret;
-}
-
-static __inline__ void
-stq_u(unsigned long val, unsigned long *p)
-{
-    unsigned long addr = (unsigned long) p;
-
-    __asm__ __volatile__("wsbh %0, %0;\n\t" "rotri %0, %0, 16;\n\t" "smw.bi %0, [%1], %0, 0;\n\t":      /* No outputs */
-                         :"r"(val), "r"(addr));
-}
-
-static __inline__ void
-stl_u(unsigned long val, unsigned int *p)
-{
-    unsigned long addr = (unsigned long) p;
-
-    __asm__ __volatile__("wsbh %0, %0;\n\t" "rotri %0, %0, 16;\n\t" "smw.bi %0, [%1], %0, 0;\n\t":      /* No outputs */
-                         :"r"(val), "r"(addr));
-}
-
 #else                           /* !NDS32_MMIO_SWAP */
 static __inline__ void
 outb(unsigned PORT_SIZE port, unsigned char val)
@@ -1273,121 +965,9 @@ inl(unsigned PORT_SIZE port)
     return *(volatile unsigned int *) (((unsigned PORT_SIZE) (port)));
 }
 
-static __inline__ unsigned long
-ldq_u(unsigned long *p)
-{
-    unsigned long addr = (unsigned long) p;
-    unsigned int ret;
-
-    __asm__ __volatile__("lmw.bi %0, [%1], %0, 0;\n\t":"=r"(ret)
-                         :"r"(addr));
-
-    return ret;
-}
-
-static __inline__ unsigned long
-ldl_u(unsigned int *p)
-{
-    unsigned long addr = (unsigned long) p;
-    unsigned int ret;
-
-    __asm__ __volatile__("lmw.bi %0, [%1], %0, 0;\n\t":"=r"(ret)
-                         :"r"(addr));
-
-    return ret;
-}
-
-static __inline__ void
-stq_u(unsigned long val, unsigned long *p)
-{
-    unsigned long addr = (unsigned long) p;
-
-    __asm__ __volatile__("smw.bi %0, [%1], %0, 0;\n\t": /* No outputs */
-                         :"r"(val), "r"(addr));
-}
-
-static __inline__ void
-stl_u(unsigned long val, unsigned int *p)
-{
-    unsigned long addr = (unsigned long) p;
-
-    __asm__ __volatile__("smw.bi %0, [%1], %0, 0;\n\t": /* No outputs */
-                         :"r"(val), "r"(addr));
-}
 #endif                          /* NDS32_MMIO_SWAP */
 
-#if (((X_BYTE_ORDER == X_BIG_ENDIAN) && !defined(NDS32_MMIO_SWAP)) || ((X_BYTE_ORDER != X_BIG_ENDIAN) && defined(NDS32_MMIO_SWAP)))
-#define ldw_u(p)	((*(unsigned char *)(p)) << 8 | \
-			(*((unsigned char *)(p)+1)))
-#define stw_u(v,p)	(*(unsigned char *)(p)) = ((v) >> 8); \
-				(*((unsigned char *)(p)+1)) = (v)
-#else
-#define ldw_u(p)	((*(unsigned char *)(p)) | \
-			(*((unsigned char *)(p)+1)<<8))
-#define stw_u(v,p)	(*(unsigned char *)(p)) = (v); \
-				(*((unsigned char *)(p)+1)) = ((v) >> 8)
-#endif
-
-#define mem_barrier()           /* XXX: nop for now */
-#define write_mem_barrier()     /* XXX: nop for now */
-
-#else                           /* ix86 */
-
-#if !defined(__SUNPRO_C)
-#if !defined(FAKEIT) && !defined(__mc68000__) && !defined(__arm__) && !defined(__sh__) && !defined(__hppa__) && !defined(__s390__) && !defined(__m32r__)
-#ifdef GCCUSESGAS
-
-/*
- * If gcc uses gas rather than the native assembler, the syntax of these
- * inlines has to be different.		DHD
- */
-
-static __inline__ void
-outb(unsigned short port, unsigned char val)
-{
-    __asm__ __volatile__("outb %0,%1"::"a"(val), "d"(port));
-}
-
-static __inline__ void
-outw(unsigned short port, unsigned short val)
-{
-    __asm__ __volatile__("outw %0,%1"::"a"(val), "d"(port));
-}
-
-static __inline__ void
-outl(unsigned short port, unsigned int val)
-{
-    __asm__ __volatile__("outl %0,%1"::"a"(val), "d"(port));
-}
-
-static __inline__ unsigned int
-inb(unsigned short port)
-{
-    unsigned char ret;
-    __asm__ __volatile__("inb %1,%0":"=a"(ret):"d"(port));
-
-    return ret;
-}
-
-static __inline__ unsigned int
-inw(unsigned short port)
-{
-    unsigned short ret;
-    __asm__ __volatile__("inw %1,%0":"=a"(ret):"d"(port));
-
-    return ret;
-}
-
-static __inline__ unsigned int
-inl(unsigned short port)
-{
-    unsigned int ret;
-    __asm__ __volatile__("inl %1,%0":"=a"(ret):"d"(port));
-
-    return ret;
-}
-
-#else                           /* GCCUSESGAS */
+#elif defined(__i386__) || defined(__ia64__)
 
 static __inline__ void
 outb(unsigned short port, unsigned char val)
@@ -1434,47 +1014,7 @@ inl(unsigned short port)
     return ret;
 }
 
-#endif                          /* GCCUSESGAS */
-
-#else                           /* !defined(FAKEIT) && !defined(__mc68000__)  && !defined(__arm__) && !defined(__sh__) && !defined(__hppa__) && !defined(__m32r__) */
-
-static __inline__ void
-outb(unsigned short port, unsigned char val)
-{
-}
-
-static __inline__ void
-outw(unsigned short port, unsigned short val)
-{
-}
-
-static __inline__ void
-outl(unsigned short port, unsigned int val)
-{
-}
-
-static __inline__ unsigned int
-inb(unsigned short port)
-{
-    return 0;
-}
-
-static __inline__ unsigned int
-inw(unsigned short port)
-{
-    return 0;
-}
-
-static __inline__ unsigned int
-inl(unsigned short port)
-{
-    return 0;
-}
-
-#endif                          /* FAKEIT */
-#endif                          /* __SUNPRO_C */
-
-#endif                          /* ix86 */
+#endif                          /* arch madness */
 
 #else                           /* !GNUC */
 #if defined(__STDC__) && (__STDC__ == 1)
@@ -1485,18 +1025,12 @@ inl(unsigned short port)
 #if !defined(__SUNPRO_C)
 #include <sys/inline.h>
 #endif
-#if !defined(__HIGHC__) && !defined(__SUNPRO_C) || \
-	defined(__USLC__)
-#pragma asm partial_optimization outl
-#pragma asm partial_optimization outw
-#pragma asm partial_optimization outb
-#pragma asm partial_optimization inl
-#pragma asm partial_optimization inw
-#pragma asm partial_optimization inb
-#endif
 #endif                          /* __GNUC__ */
 
-#endif                          /* NO_INLINE */
+#if !defined(MMIO_IS_BE) && \
+    (defined(SPARC_MMIO_IS_BE) || defined(PPC_MMIO_IS_BE))
+#define MMIO_IS_BE
+#endif
 
 #ifdef __alpha__
 /* entry points for Mmio memory access routines */
@@ -1517,9 +1051,6 @@ xf86ReadMmio32(void *Base, unsigned long Offset)
 extern _X_EXPORT void (*xf86WriteMmio8) (int, void *, unsigned long);
 extern _X_EXPORT void (*xf86WriteMmio16) (int, void *, unsigned long);
 extern _X_EXPORT void (*xf86WriteMmio32) (int, void *, unsigned long);
-extern _X_EXPORT void (*xf86WriteMmioNB8) (int, void *, unsigned long);
-extern _X_EXPORT void (*xf86WriteMmioNB16) (int, void *, unsigned long);
-extern _X_EXPORT void (*xf86WriteMmioNB32) (int, void *, unsigned long);
 extern _X_EXPORT void xf86SlowBCopyFromBus(unsigned char *, unsigned char *,
                                            int);
 extern _X_EXPORT void xf86SlowBCopyToBus(unsigned char *, unsigned char *, int);
@@ -1539,43 +1070,29 @@ extern _X_EXPORT void xf86SlowBCopyToBus(unsigned char *, unsigned char *, int);
 	write_mem_barrier(); \
 	*(volatile CARD32 *)(void *)(((CARD8*)(base)) + (offset)) = (val); \
     } while (0)
-#define MMIO_ONB32(base, offset, val) \
-	*(volatile CARD32 *)(void *)(((CARD8*)(base)) + (offset)) = (val)
 
 #define MMIO_OUT8(base, offset, val) \
     (*xf86WriteMmio8)((CARD8)(val), base, offset)
 #define MMIO_OUT16(base, offset, val) \
     (*xf86WriteMmio16)((CARD16)(val), base, offset)
-#define MMIO_ONB8(base, offset, val) \
-    (*xf86WriteMmioNB8)((CARD8)(val), base, offset)
-#define MMIO_ONB16(base, offset, val) \
-    (*xf86WriteMmioNB16)((CARD16)(val), base, offset)
-#define MMIO_MOVE32(base, offset, val) \
-    MMIO_OUT32(base, offset, val)
 
-#elif defined(__powerpc__)
+#elif defined(__powerpc__) || defined(__sparc__)
  /* 
   * we provide byteswapping and no byteswapping functions here
   * with byteswapping as default, 
-  * drivers that don't need byteswapping should define PPC_MMIO_IS_BE 
+  * drivers that don't need byteswapping should define MMIO_IS_BE
   */
 #define MMIO_IN8(base, offset) xf86ReadMmio8(base, offset)
 #define MMIO_OUT8(base, offset, val) \
     xf86WriteMmio8(base, offset, (CARD8)(val))
-#define MMIO_ONB8(base, offset, val) \
-    xf86WriteMmioNB8(base, offset, (CARD8)(val))
 
-#if defined(PPC_MMIO_IS_BE)     /* No byteswapping */
+#if defined(MMIO_IS_BE)     /* No byteswapping */
 #define MMIO_IN16(base, offset) xf86ReadMmio16Be(base, offset)
 #define MMIO_IN32(base, offset) xf86ReadMmio32Be(base, offset)
 #define MMIO_OUT16(base, offset, val) \
     xf86WriteMmio16Be(base, offset, (CARD16)(val))
 #define MMIO_OUT32(base, offset, val) \
     xf86WriteMmio32Be(base, offset, (CARD32)(val))
-#define MMIO_ONB16(base, offset, val) \
-    xf86WriteMmioNB16Be(base, offset, (CARD16)(val))
-#define MMIO_ONB32(base, offset, val) \
-    xf86WriteMmioNB32Be(base, offset, (CARD32)(val))
 #else                           /* byteswapping is the default */
 #define MMIO_IN16(base, offset) xf86ReadMmio16Le(base, offset)
 #define MMIO_IN32(base, offset) xf86ReadMmio32Le(base, offset)
@@ -1583,56 +1100,8 @@ extern _X_EXPORT void xf86SlowBCopyToBus(unsigned char *, unsigned char *, int);
      xf86WriteMmio16Le(base, offset, (CARD16)(val))
 #define MMIO_OUT32(base, offset, val) \
      xf86WriteMmio32Le(base, offset, (CARD32)(val))
-#define MMIO_ONB16(base, offset, val) \
-     xf86WriteMmioNB16Le(base, offset, (CARD16)(val))
-#define MMIO_ONB32(base, offset, val) \
-     xf86WriteMmioNB32Le(base, offset, (CARD32)(val))
 #endif
 
-#define MMIO_MOVE32(base, offset, val) \
-       xf86WriteMmio32Be(base, offset, (CARD32)(val))
-
-#elif defined(__sparc__) || defined(sparc) || defined(__sparc)
- /*
-  * Like powerpc, we provide byteswapping and no byteswapping functions
-  * here with byteswapping as default, drivers that don't need byteswapping
-  * should define SPARC_MMIO_IS_BE (perhaps create a generic macro so that we
-  * do not need to use PPC_MMIO_IS_BE and the sparc one in all the same places
-  * of drivers?).
-  */
-#define MMIO_IN8(base, offset) xf86ReadMmio8(base, offset)
-#define MMIO_OUT8(base, offset, val) \
-    xf86WriteMmio8(base, offset, (CARD8)(val))
-#define MMIO_ONB8(base, offset, val) \
-    xf86WriteMmio8NB(base, offset, (CARD8)(val))
-
-#if defined(SPARC_MMIO_IS_BE)   /* No byteswapping */
-#define MMIO_IN16(base, offset) xf86ReadMmio16Be(base, offset)
-#define MMIO_IN32(base, offset) xf86ReadMmio32Be(base, offset)
-#define MMIO_OUT16(base, offset, val) \
-     xf86WriteMmio16Be(base, offset, (CARD16)(val))
-#define MMIO_OUT32(base, offset, val) \
-     xf86WriteMmio32Be(base, offset, (CARD32)(val))
-#define MMIO_ONB16(base, offset, val) \
-     xf86WriteMmio16BeNB(base, offset, (CARD16)(val))
-#define MMIO_ONB32(base, offset, val) \
-     xf86WriteMmio32BeNB(base, offset, (CARD32)(val))
-#else                           /* byteswapping is the default */
-#define MMIO_IN16(base, offset) xf86ReadMmio16Le(base, offset)
-#define MMIO_IN32(base, offset) xf86ReadMmio32Le(base, offset)
-#define MMIO_OUT16(base, offset, val) \
-     xf86WriteMmio16Le(base, offset, (CARD16)(val))
-#define MMIO_OUT32(base, offset, val) \
-     xf86WriteMmio32Le(base, offset, (CARD32)(val))
-#define MMIO_ONB16(base, offset, val) \
-     xf86WriteMmio16LeNB(base, offset, (CARD16)(val))
-#define MMIO_ONB32(base, offset, val) \
-     xf86WriteMmio32LeNB(base, offset, (CARD32)(val))
-#endif
-
-#define MMIO_MOVE32(base, offset, val) \
-       xf86WriteMmio32Be(base, offset, (CARD32)(val))
-
 #elif defined(__nds32__)
  /*
   * we provide byteswapping and no byteswapping functions here
@@ -1643,8 +1112,6 @@ extern _X_EXPORT void xf86SlowBCopyToBus(unsigned char *, unsigned char *, int);
 #define MMIO_IN8(base, offset) xf86ReadMmio8(base, offset)
 #define MMIO_OUT8(base, offset, val) \
     xf86WriteMmio8(base, offset, (CARD8)(val))
-#define MMIO_ONB8(base, offset, val) \
-    xf86WriteMmioNB8(base, offset, (CARD8)(val))
 
 #if defined(NDS32_MMIO_SWAP)    /* byteswapping */
 #define MMIO_IN16(base, offset) xf86ReadMmio16Swap(base, offset)
@@ -1653,10 +1120,6 @@ extern _X_EXPORT void xf86SlowBCopyToBus(unsigned char *, unsigned char *, int);
     xf86WriteMmio16Swap(base, offset, (CARD16)(val))
 #define MMIO_OUT32(base, offset, val) \
     xf86WriteMmio32Swap(base, offset, (CARD32)(val))
-#define MMIO_ONB16(base, offset, val) \
-    xf86WriteMmioNB16Swap(base, offset, (CARD16)(val))
-#define MMIO_ONB32(base, offset, val) \
-    xf86WriteMmioNB32Swap(base, offset, (CARD32)(val))
 #else                           /* no byteswapping is the default */
 #define MMIO_IN16(base, offset) xf86ReadMmio16(base, offset)
 #define MMIO_IN32(base, offset) xf86ReadMmio32(base, offset)
@@ -1664,30 +1127,6 @@ extern _X_EXPORT void xf86SlowBCopyToBus(unsigned char *, unsigned char *, int);
      xf86WriteMmio16(base, offset, (CARD16)(val))
 #define MMIO_OUT32(base, offset, val) \
      xf86WriteMmio32(base, offset, (CARD32)(val))
-#define MMIO_ONB16(base, offset, val) \
-     xf86WriteMmioNB16(base, offset, (CARD16)(val))
-#define MMIO_ONB32(base, offset, val) \
-     xf86WriteMmioNB32(base, offset, (CARD32)(val))
-#endif
-
-#define MMIO_MOVE32(base, offset, val) \
-       xf86WriteMmio32(base, offset, (CARD32)(val))
-
-#ifdef N1213_HC                 /* for NDS32 N1213 hardcore */
-static __inline__ void
-nds32_flush_icache(char *addr)
-{
-    __asm__ volatile ("isync %0;"
-                      "msync;"
-                      "isb;"
-                      "cctl %0,L1I_VA_INVAL;" "isb;"::"r" (addr):"memory");
-}
-#else
-static __inline__ void
-nds32_flush_icache(char *addr)
-{
-    __asm__ volatile ("isync %0;" "isb;"::"r" (addr):"memory");
-}
 #endif
 
 #else                           /* !__alpha__ && !__powerpc__ && !__sparc__ */
@@ -1704,11 +1143,6 @@ nds32_flush_icache(char *addr)
 	*(volatile CARD16 *)(void *)(((CARD8*)(base)) + (offset)) = (val)
 #define MMIO_OUT32(base, offset, val) \
 	*(volatile CARD32 *)(void *)(((CARD8*)(base)) + (offset)) = (val)
-#define MMIO_ONB8(base, offset, val) MMIO_OUT8(base, offset, val)
-#define MMIO_ONB16(base, offset, val) MMIO_OUT16(base, offset, val)
-#define MMIO_ONB32(base, offset, val) MMIO_OUT32(base, offset, val)
-
-#define MMIO_MOVE32(base, offset, val) MMIO_OUT32(base, offset, val)
 
 #endif                          /* __alpha__ */
 
