@@ -57,24 +57,24 @@ extern _X_EXPORT int dixLookupProperty(PropertyPtr * /*result */ ,
                                        ClientPtr /*pClient */ ,
                                        Mask /*access_mode */ );
 
-extern _X_EXPORT int dixChangeWindowProperty(ClientPtr /*pClient */ ,
-                                             WindowPtr /*pWin */ ,
-                                             Atom /*property */ ,
-                                             Atom /*type */ ,
-                                             int /*format */ ,
-                                             int /*mode */ ,
-                                             unsigned long /*len */ ,
-                                             pointer /*value */ ,
-                                             Bool /*sendevent */ );
+extern _X_EXPORT int dixChangeWindowProperty(ClientPtr pClient,
+                                             WindowPtr pWin,
+                                             Atom property,
+                                             Atom type,
+                                             int format,
+                                             int mode,
+                                             unsigned long len,
+                                             void *value,
+                                             Bool sendevent);
 
-extern _X_EXPORT int ChangeWindowProperty(WindowPtr /*pWin */ ,
-                                          Atom /*property */ ,
-                                          Atom /*type */ ,
-                                          int /*format */ ,
-                                          int /*mode */ ,
-                                          unsigned long /*len */ ,
-                                          pointer /*value */ ,
-                                          Bool /*sendevent */ );
+extern _X_EXPORT int ChangeWindowProperty(WindowPtr pWin,
+                                          Atom property,
+                                          Atom type,
+                                          int format,
+                                          int mode,
+                                          unsigned long len,
+                                          void *value,
+                                          Bool sendevent);
 
 extern _X_EXPORT int DeleteProperty(ClientPtr /*client */ ,
                                     WindowPtr /*pWin */ ,
